package grizzly;

import grizzly.protocol.EchoFilter_res;
import grizzly.protocol.ReadFilter_res;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.concurrent.CountDownLatch;

import com.sun.grizzly.Controller;
import com.sun.grizzly.ControllerStateListenerAdapter;
import com.sun.grizzly.DefaultPipeline;
import com.sun.grizzly.DefaultProtocolChain;
import com.sun.grizzly.DefaultProtocolChainInstanceHandler;
import com.sun.grizzly.Pipeline;
import com.sun.grizzly.ProtocolChain;
import com.sun.grizzly.TCPSelectorHandler;
import com.sun.grizzly.connectioncache.server.CacheableSelectionKeyHandler;
import com.sun.grizzly.utils.ControllerUtils;

public class TestEchoServer {
	private static final int SERVER_PORT = 11113;
	// private static final String SERVER_HOST = "10.140.2.100";
	private Controller controller;
	protected CountDownLatch controllerReadyLatch = new CountDownLatch(1);

	public TestEchoServer(String host, int port) throws UnknownHostException {
		TCPSelectorHandler selectorHandler = new TCPSelectorHandler();
		selectorHandler.setInet(InetAddress.getByName(host));
		selectorHandler.setPort(port);
		selectorHandler.setReuseAddress(true);
		selectorHandler.setServerTimeout(30 * 60 * 1000);
		selectorHandler.setTcpNoDelay(true);

		controller = createController();

		CacheableSelectionKeyHandler cacheableKeyHandler = new CacheableSelectionKeyHandler(
				30000, 100);
		selectorHandler.setSelectionKeyHandler(cacheableKeyHandler);
		cacheableKeyHandler.setTimeout(30 * 60 * 1000);
		controller.addSelectorHandler(selectorHandler);

		controller.addStateListener(new ControllerStateListenerAdapter() {
			@Override
			public void onReady() {
				controllerReadyLatch.countDown();
			}
		});

		controller
				.setProtocolChainInstanceHandler(new DefaultProtocolChainInstanceHandler() {
					private ProtocolChain protocolChain;

					@Override
					public ProtocolChain poll() {
						if (protocolChain == null) {
							protocolChain = new DefaultProtocolChain();
							protocolChain.addFilter(new ReadFilter_res());
							protocolChain.addFilter(new EchoFilter_res());
						}
						return protocolChain;
					}
				});

		Pipeline pl = new DefaultPipeline();
		pl.setMaxThreads(100);
		controller.setPipeline(pl);
	}

	private Controller createController() {
		Controller tmpController = new Controller();
		return tmpController;
	}

	public void start() {
		ControllerUtils.startController(controller);
	}

	/**
	 * @param args
	 * @throws UnknownHostException
	 */
	public static void main(String[] args) throws UnknownHostException {
		if (args.length < 1) {
			usage();
		}
		TestEchoServer server = new TestEchoServer(args[0], SERVER_PORT);
		server.start();
	}

	private static void usage() {
		System.out
				.println("Usage: java tests.grizzly.TestEchoServer {binding_host}");
		System.exit(0);
	}

}
