package grizzly.protocol;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.SocketAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SelectionKey;
import java.util.Queue;

import com.sun.grizzly.CallbackHandler;
import com.sun.grizzly.Context;
import com.sun.grizzly.Controller;
import com.sun.grizzly.DefaultProtocolChain;
import com.sun.grizzly.DefaultSelectionKeyHandler;
import com.sun.grizzly.IOEvent;
import com.sun.grizzly.ProtocolChain;
import com.sun.grizzly.ProtocolChainInstanceHandler;
import com.sun.grizzly.ProtocolFilter;
import com.sun.grizzly.TCPConnectorHandler;
import com.sun.grizzly.TCPSelectorHandler;
import com.sun.grizzly.Context.KeyRegistrationState;
import com.sun.grizzly.Controller.Protocol;
import com.sun.grizzly.async.AsyncReadCallbackHandler;
import com.sun.grizzly.async.AsyncReadQueueRecord;
import com.sun.grizzly.util.OutputWriter;
import com.sun.grizzly.util.WorkerThread;
import com.sun.grizzly.utils.ControllerUtils;

public class TestProtocolFilter10 {
	private static final int SERVER_PORT = 11113;
	private static final String SERVER_HOST_OR_IP = "localhost";
	private static final int keepAliveTimeoutInSeconds = 60;
	private Controller controller;
	private static int requestCount = 0;

	public void initController() throws UnknownHostException {
		controller = new Controller();

		TCPSelectorHandler selectorHandler = new TCPSelectorHandler(true);
		// selectorHandler.setPort(SERVER_PORT);
		// selectorHandler.setInet(InetAddress.getByName(SERVER_HOST_OR_IP));
		// selectorHandler.setLinger(linger);
		// selectorHandler.setLogger(logger);
		selectorHandler.setReuseAddress(true);
		// selectorHandler.setSelectTimeout(selectorTimeout);
		// selectorHandler.setServerTimeout(serverTimeout);
		// selectorHandler.setSsBackLog(ssBackLog);
		selectorHandler.setTcpNoDelay(true);
		controller.setSelectorHandler(selectorHandler);

		DefaultSelectionKeyHandler keyHandler = new DefaultSelectionKeyHandler();
		// keyHandler.setLogger(logger);
		keyHandler.setTimeout(keepAliveTimeoutInSeconds * 1000);
		controller.setSelectionKeyHandler(keyHandler);

		ProtocolChainInstanceHandler instanceHandler = new ProtocolChainInstanceHandler() {
			private ProtocolChain protocolChain;
			private ProtocolFilter requestFilter = new ProtocolFilter() {

				public boolean execute(Context ctx) throws IOException {
					ctx.setCurrentOpType(Context.OpType.OP_WRITE);
					System.out
							.println("requestFilter execute():ctx.getCurrentOpType()="
									+ ctx.getCurrentOpType());
					ByteBuffer byteBuffer = ((WorkerThread) Thread
							.currentThread()).getByteBuffer();
					if (ctx.getCurrentOpType() != Context.OpType.OP_WRITE) {
						System.out
								.println("error state: not OP_WRITE in requestFilter");
						return false;
					}

					SelectableChannel channel = ctx.getSelectionKey().channel();
					OutputWriter.flushChannel(channel, ByteBuffer
							.wrap("testing".getBytes()));
					ctx.setAttribute("REQUEST_COUNT", ++requestCount);

					ctx.setCurrentOpType(Context.OpType.OP_READ);

					// ctx
					// .setKeyRegistrationState(Context.KeyRegistrationState.REGISTER);

					// {Add your logic here}
					// (1) client need a response
					// Flush the response and then add which will register the
					// key back to the Selector for new request:
					// ctx
					// .setKeyRegistrationState(Context.KeyRegistrationState.REGISTER);

					// Tell Grizzly to not do ANYTHING with the key
					// This is needed to support Asynchonous Request
					// Processing
					// client ---> request ----> server ---> wait until
					// response
					// Here you need to have add a Classback so when the
					// server is ready to push data back to the client,
					// you can resume the connection.
					// ctx
					// .setKeyRegistrationState(Context.KeyRegistrationState.NONE);
					Integer rc = (Integer) ctx.getAttribute("REQUEST_COUNT");
					System.out.println("rc=" + rc.intValue());

					SelectionKey key = ctx.getSelectionKey();
					key.interestOps(key.interestOps() & ~SelectionKey.OP_WRITE);

					return rc.intValue() == 5 ? false : true;
				}

				// public void callBack(ServerCallback serverCallback) {
				// // Push something to the client
				//
				// // Tell the server you are ready for more client
				// requests.
				// controller.getSelectionKeyHandler().register(key);
				// }

				public boolean postExecute(Context ctx) throws IOException {
					System.out.println("requestFilter.postExecute()");
					SelectableChannel channel = ctx.getSelectionKey().channel();
					ctx.getSelectorHandler().register(channel,
							SelectionKey.OP_READ);

					return true;
				}

			};
			private ProtocolFilter responseFilter = new ProtocolFilter() {
				public boolean execute(Context ctx) throws IOException {
					ctx.setCurrentOpType(Context.OpType.OP_READ);
					ByteBuffer buffer = ByteBuffer.allocate(100);
					AsyncReadCallbackHandler callbackHandler = new AsyncReadCallbackHandler() {
						public void onIOException(IOException ioException,
								SelectionKey key, ByteBuffer buffer,
								Queue<AsyncReadQueueRecord> remainingQueue) {
							// TODO Auto-generated method stub

						}

						public void onReadCompleted(SelectionKey key,
								SocketAddress srcAddress, ByteBuffer buffer) {
							System.out.println("onReadComplete()");
						}
					};
					ctx.getAsyncQueueReadable().readFromAsyncQueue(buffer,
							callbackHandler);
					SelectionKey key = ctx.getSelectionKey();
					key.interestOps(key.interestOps() & ~SelectionKey.OP_READ);
					ctx.setKeyRegistrationState(KeyRegistrationState.NONE);
					return false;
				}

				public boolean postExecute(Context ctx) throws IOException {
					// TODO Auto-generated method stub
					return false;
				}
			};

			/**
			 * Always return instance of ProtocolChain.
			 */
			public ProtocolChain poll() {
				if (protocolChain == null) {
					protocolChain = new DefaultProtocolChain();
					// protocolChain.addFilter(new ReadFilter());
					protocolChain.addFilter(requestFilter);
					protocolChain.addFilter(responseFilter);
				}
				return protocolChain;
			}

			/**
			 * Pool an instance of ProtocolChain.
			 */
			public boolean offer(ProtocolChain instance) {
				return true;
			}
		};
		controller.setProtocolChainInstanceHandler(instanceHandler);
	}

	/**
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		TestProtocolFilter10 tpf = new TestProtocolFilter10();
		tpf.initController();
		tpf.start();
	}

	private void start() throws IOException {
		ControllerUtils.startController(controller);

		final TCPConnectorHandler ch = (TCPConnectorHandler) controller
				.acquireConnectorHandler(Protocol.TCP);
		CallbackHandler<Context> connectCallback = new CallbackHandler<Context>() {

			public void onConnect(IOEvent<Context> ioEvent) {
				System.out.println("onConnect()");
				try {
					ch.finishConnect(ioEvent.attachment().getSelectionKey());
					Context ctx = ioEvent.attachment();
					// ctx.setCurrentOpType(Context.OpType.OP_WRITE);
					ctx.getProtocolChain().execute(ctx);
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}

			public void onRead(IOEvent<Context> ioEvent) {
				// TODO Auto-generated method stub

			}

			public void onWrite(IOEvent<Context> ioEvent) {
				// TODO Auto-generated method stub

			}
		};
		ch.connect(new InetSocketAddress(SERVER_HOST_OR_IP, SERVER_PORT),
				connectCallback);
	}

}
